@@ -1,64 +1,20 @@
 package ExtUtils::HasCompiler;
-$ExtUtils::HasCompiler::VERSION = '0.004';
+$ExtUtils::HasCompiler::VERSION = '0.005';
 use strict;
 use warnings;
 
-use Exporter 5.57 'import';
-our @EXPORT_OK = qw/can_compile_executable can_compile_loadable_object/;
+use base 'Exporter';
+our @EXPORT_OK = qw/can_compile_loadable_object/;
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
 
 use Config;
 use Carp 'croak';
 use File::Basename 'basename';
-use File::Spec::Functions qw/curdir catfile catdir rel2abs/;
+use File::Spec::Functions qw/catfile catdir/;
 use File::Temp qw/tempdir tempfile/;
 
 my $tempdir = tempdir(CLEANUP => 1);
 
-sub _write_file {
-	my ($fh, $content) = @_;
-	print $fh $content or croak "Couldn't write to file: $!";
-	close $fh or croak "Couldn't close file: $!";
-}
-
-my $executable_code = <<'END';
-#include <stdlib.h>
-#include <stdio.h>
-
-int main(int argc, char** argv) {
-	puts("It seems we've got a working compiler");
-	return 0;
-}
-END
-
-sub can_compile_executable {
-	my %args = @_;
-
-	my ($source_handle, $source_name) = tempfile(DIR => $tempdir, SUFFIX => '.c');
-	_write_file($source_handle, $executable_code);
-
-	my $config = $args{config} || 'ExtUtils::HasCompiler::Config';
-	my ($cc, $ccflags, $ldflags, $libs) = map { $config->get($_) } qw/cc ccflags ldflags libs/;
-	my $executable = catfile($tempdir, basename($source_name, '.c') . $config->get('_exe'));
-
-	my $command;
-	if ($^O eq 'MSWin32' && $config->get('cc') =~ /^cl/) {
-		$command = "$cc $ccflags -Fe$executable $source_name -link $ldflags $libs";
-	}
-	elsif ($^O eq 'VMS') {
-		warn "VMS is currently unsupported";
-		return;
-	}
-	else {
-		# Assume UNIXish
-		$command = "$cc $ccflags -o $executable $source_name $ldflags";
-	}
-
-	print "$command\n" if not $args{quiet};
-	system $command and return;
-	return not system(rel2abs($executable));
-}
-
 my $loadable_object_format = <<'END';
 #define PERL_NO_GET_CONTEXT
 #include "EXTERN.h"
@@ -107,16 +63,18 @@ my $counter = 1;
 sub can_compile_loadable_object {
 	my %args = @_;
 
+	my $config = $args{config} || 'ExtUtils::HasCompiler::Config';
+	return if not $config->get('usedl');
+
 	my ($source_handle, $source_name) = tempfile(DIR => $tempdir, SUFFIX => '.c', UNLINK => 1);
 	my $basename = basename($source_name, '.c');
 
 	my $shortname = '_Loadable' . $counter++;
 	my $package = "ExtUtils::HasCompiler::$shortname";
-	my $loadable_object_code = sprintf $loadable_object_format, $basename, $package;
-	_write_file($source_handle, $loadable_object_code);
+	printf $source_handle $loadable_object_format, $basename, $package or croak "Couldn't write to file: $!";
+	close $source_handle or croak "Couldn't close sourcefile: $!";
 
-	my $config = $args{config} || 'ExtUtils::HasCompiler::Config';
-	my ($cc, $ccflags, $optimize, $cccdlflags, $lddlflags, $perllibs, $archlibexp) = map { $config->get($_) } qw/cc ccflags optimize cccdlflags lddlflags perllibs archlibexp/;
+	my ($cc, $ccflags, $optimize, $cccdlflags, $lddlflags, $libperl, $perllibs, $archlibexp) = map { $config->get($_) } qw/cc ccflags optimize cccdlflags lddlflags libperl perllibs archlibexp/;
 	my $incdir = catdir($archlibexp, 'CORE');
 
 	my $loadable_object = catfile($tempdir, $basename . '.' . $config->get('dlext'));
@@ -127,18 +85,19 @@ sub can_compile_loadable_object {
 		my $abs_basename = catfile($tempdir, $basename);
 		#Mksymlists will add the ext on its own
 		ExtUtils::Mksymlists::Mksymlists(NAME => $basename, FILE => $abs_basename);
-		$command = qq{$cc $ccflags $optimize /I "$incdir" $source_name $abs_basename.def /Fo$abs_basename.obj /Fd$abs_basename.pdb /link $lddlflags $perllibs /out:$loadable_object};
+		$command = qq{$cc $ccflags $optimize /I "$incdir" $source_name $abs_basename.def /Fo$abs_basename.obj /Fd$abs_basename.pdb /link $lddlflags $libperl $perllibs /out:$loadable_object};
 	}
 	elsif ($^O eq 'VMS') {
 		warn "VMS is currently unsupported";
 		return;
 	}
 	else {
+		my $extra = $^O eq 'MSWin32' ? $libperl : '';
 		if ($^O eq 'aix') {
 			$lddlflags =~ s/\Q$(BASEEXT)\E/$basename/;
 			$lddlflags =~ s/\Q$(PERL_INC)\E/$incdir/;
 		}
-		$command = qq{$cc $ccflags "-I$incdir" $cccdlflags $source_name $lddlflags $perllibs -o $loadable_object };
+		$command = qq{$cc $ccflags "-I$incdir" $cccdlflags $source_name $lddlflags $extra $perllibs -o $loadable_object };
 	}
 
 	print "$command\n" if not $args{quiet};
@@ -181,20 +140,16 @@ ExtUtils::HasCompiler - Check for the presence of a compiler
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 DESCRIPTION
 
-This module tries to thorougly check if the current system has a working compiler.
+This module tries to check if the current system is capable of compiling, linking and loading an XS module.
 
 B<Notice>: this is an early release, interface stability isn't guaranteed yet.
 
 =head1 FUNCTIONS
 
-=head2 can_compile_executable(%opts)
-
-This checks if the system can compile and link an executable. This may be removed in the future.
-
 =head2 can_compile_loadable_object(%opts)
 
 This checks if the system can compile, link and load a perl loadable object. It may take the following options:
